#!/bin/bash

set -e

echo %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
echo % QPALMA Example: get_data.sh %
echo % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
echo 
echo This scripts obtains reads, the genome sequence 
echo and splice site predictions for the C. elegans
echo genome.
echo 

export dataDir=data

if [ -d $dataDir ]
then
	echo Data directory ./$dataDir already exists
	echo \(remove it to run this script again\)
	exit -1
fi
mkdir $dataDir

echo Downloading reads in fastq format from the Short Read Archive ...
wget -c ftp://ftp.tuebingen.mpg.de/pub/fml/raetsch-lab/software/qpalma/data/SRX001872_read_sample.fastq.txt.gz
echo uncompressing ...
gzip -d SRX001872_read_sample.fastq.txt.gz
mv SRX001872_read_sample.fastq.txt $dataDir
echo -n Done.
echo

echo Downloading C. elegans WS209 annotation used for generating artificial reads
cd data
wget ftp://ftp.wormbase.org:/pub/wormbase/genomes/c_elegans/genome_feature_tables/GFF3/c_elegans.WS209.gff3.gz
gzip -d c_elegans.WS209.gff3.gz
#ln -s c_elegans.WS209.gff3 c_elegans.WS200.gff3
cd ..

echo Downloading splice site predictions and genome ...
wget -c ftp://ftp.tuebingen.mpg.de/pub/fml/raetsch-lab/predictions/splice/C_elegans/C_elegans_SpliceSitePred_WS209.tar.bz2
echo uncompresssing ...
cd $dataDir 
tar xf ../C_elegans_SpliceSitePred_WS209.tar.bz2 
rm ../C_elegans_SpliceSitePred_WS209.tar.bz2 
cd ..
bzip2 -d $dataDir/C_elegans_SpliceSitePred_WS209/genome/C_elegans_WS209_genome.fasta.bz2
mv $dataDir/C_elegans_SpliceSitePred_WS209/genome $dataDir
echo -n Done.
echo

