#ifndef _QPALMA_DP_H_
#define _QPALMA_DP_H_

#include "Python.h"

#include "penalty_info.h"
#include "debug_tools.h"

struct ArrayElem { //24B
  double score;
  int prev_j;
  int prev_i;
  int prev_matrix_no;   
  bool isSplice;
} ;

struct pre_score { //24B
  double value; //8
  int prev_i; //4
  int prev_j; //4
  int prev_matrix_no;//4
  bool issplice;//4
};

typedef struct pre_score Pre_score;

typedef struct Align_pair { //24B
  double value ; 
  bool issplice;
  int previ,prevj ;
  char est_char;
  char dna_char;
};

void fill_matrix (int nr_paths, Pre_score* matrices[], int len_est, int len_dna, char* est, char* dna, double* prb, penalty_struct* functions, double* matchmatrix, penalty_struct* qualityScores, double* donor, double* acceptor, bool remove_duplicate_scores, int nr_donor_sites, int* donor_sites, int* max_score_positions);

int check_char(char base);

bool result_align (Pre_score* matrices[], int matrixnr, int i,  int j, int* result_length_ptr, char* est, char* dna, double* prb, double* chastity,  int* s_align, int* e_align, int* m_param, double* alignmentscores, int* max_score_positions, penalty_struct* qparam, mode currentMode);

extern void print_align(Pre_score* matrix, int length_est,  int length_dna, Align_pair* vektor, int result_length, int print_matrix);


/* 
 * The function myalign calculates a new alignment given the scoring scheme
 * Its input arguments are:
 *
 * num_path(id)               ->    an integer specifying which alignment has to be done
 * dna                        ->    
 * est                        ->
 * h                          ->
 * mmatrix                    -> 
 * donor                      ->
 * acceptor                   ->
 * remove_duplicate_scores    -> a boolean
 * print_matrix               -> a boolean
 *
 * [SpliceAlign, EstAlign, weightMatch, Gesamtscores, dnaest] = myalign_local(...
 *
 * the idea of the qualityScores array is as follows
 *
 * consider a matrix of 24 plifs
 * 
 * -> row major
 *
 */

class Alignment {

   private:
      int* splice_align;
      int* est_align;
      int* mmatrix_param;
      double* alignmentscores;
      struct penalty_struct** qualityFeaturesAllPaths;

      size_t dna_len;
      size_t est_len;
      size_t mlen;
      size_t nr_paths;

      size_t result_len;
      int* DNA_ARRAY;
      int* EST_ARRAY;

      int numQualSuppPoints;
      int numPlifs;
      bool use_quality_scores;

      uint32_t splice_align_size ;
      uint32_t est_align_size ;
      uint32_t mmatrix_param_size ;
      uint32_t alignmentscores_size ;
      uint32_t numPathsPlifs ;

      INT len;
      REAL *limits;
      REAL *penalties;
      INT max_len;
      INT min_len;
      REAL *cache;
      enum ETransformType transform ;
      INT id;
      char * name;
      INT use_svm;

   public:
      Alignment(int numQPlifs ,int numq, bool use_qscores);
      ~Alignment() {
         if(splice_align != 0)
            delete[] splice_align;

         if(est_align != 0)
            delete[] est_align;

         if(mmatrix_param != 0)
            delete[] mmatrix_param;

         if(alignmentscores != 0)
            delete[] alignmentscores;

         if(qualityFeaturesAllPaths != 0)
            delete[] qualityFeaturesAllPaths;
      }

      void myalign(int nr_paths_p, char* dna, int dna_len_p, char* est,
      int est_len_p, double* prb, double* chastity, struct penalty_struct h, 
      double* matchmatrix, int mm_len, double* donor, int d_len, double* acceptor,
      int a_len, struct penalty_struct* qualityScores, bool remove_duplicate_scores,
      bool print_matrix);

      void getDNAEST();

      int getResultLength() { return result_len; }

      void getAlignmentResults(int* s_align, int* e_align,
      int* mmatrix_p, double* alignscores, double* qScores);
    
      void getAlignmentArrays(int* dna_align, int* est_align);


      PyObject* getAlignmentResultsNew();
      PyObject* getAlignmentArraysNew();
};

#endif  // _QPALMA_DP_H_
