/*

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Written (W) 2008 Fabio De Bona, Andre Noll, Gunnar Raetsch
Copyright (C) 2008 Max-Planck-Society

*/

%module IntervalQuery
%{

#include "IntervalQuery.h"

%}

%include "carrays.i"

%array_class(int,intArray)
%array_class(double,doubleArray)

%include "typemaps.i"

%include "cpointer.i"
%pointer_functions(int,intp)
%pointer_functions(double,doublep)


%typemap(in) char ** {
  /* Check if is a list */
  if (PyList_Check($input)) {
    int size = PyList_Size($input);
    int i = 0;
    $1 = (char **) malloc((size+1)*sizeof(char *));
    for (i = 0; i < size; i++) {
      PyObject *o = PyList_GetItem($input,i);
      if (PyString_Check(o))
   $1[i] = PyString_AsString(PyList_GetItem($input,i));
      else {
   PyErr_SetString(PyExc_TypeError,"list must contain strings");
   free($1);
   return NULL;
      }
    }
    $1[i] = 0;
  } else {
    PyErr_SetString(PyExc_TypeError,"not a list");
    return NULL;
  }
}

%include "IntervalQuery.h"

%pythoncode
%{

def createDoubleArrayFromList(list):
     #array = new_doubleArray(len(list))
     array = doubleArray(len(list))
     for i in range(len(list)):
     #   doubleArray_setitem(array, i, list[i])
        array[i] = list[i]
     return array
  
def createIntArrayFromList(list):
     #array = new_intArray(len(list))
     array = intArray(len(list))
     for i in range(len(list)):
        #intArray_setitem(array, i, list[i])
        array[i] = list[i]
     return array

def doIntervalQuery(chr,strand,_intervalBegin,_intervalEnd,file_expr,total_size=0):

   from numpy import inf
   import pdb

   intervalBegin  = _intervalBegin
   intervalEnd    = _intervalEnd

   #if strand == '-':
   #   intervalBegin  = total_size - _intervalEnd + 1 
   #   intervalEnd    = total_size - _intervalBegin + 1

   assert intervalBegin >= 0 and intervalEnd >= 0, 'interval boundaries should be non-negative'
   assert intervalEnd > intervalBegin, 'error interval is of negative size'
   size = intervalEnd-intervalBegin
   #print 'interval is [%d,%d] (size: %d)' % (intervalBegin,intervalEnd,size)

   interval_matrix = createIntArrayFromList([intervalBegin,intervalEnd])

   iquery = IntervalQuery()
   num_fields     = 1
   num_intervals  = 1
   #num_hits = iquery.query(file_expr%(chr,strand), ['Conf'], num_fields, interval_matrix, num_intervals)
   #print file_expr
   #print 'calling iquery...'
   num_hits = iquery.query(file_expr, ['Conf_cum'], num_fields, interval_matrix, num_intervals)
   #print 'iquery success'

   c_pos      = createIntArrayFromList([0]*num_hits)
   c_indices  = createIntArrayFromList([0]*num_hits)
   c_scores   = createDoubleArrayFromList([0]*num_hits*num_fields)

   iquery.getResults(c_pos,c_indices,c_scores)
   del iquery

   return_scores = [-inf]*(size)

   for i in range(num_hits):
      position = c_pos[i] - intervalBegin

      #if strand == '-':
      #   position = total_size - c_pos[i] - intervalBegin - 1

      if not (position > 1 and position < size):
         continue
  
      assert 0 <= position <= size, pdb.set_trace()
      return_scores[position] = c_scores[i]

   if strand == '-':
      return_scores.reverse()
   
   return return_scores
%}
