#include "CParaParser.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/stat.h>

/*
 * The constructor needs the format string to be used with sscanf and the names
 * of the respective fields for the dictionary.
 *
 */

ParaParser::ParaParser(const char* fmt, const char** fields) {
   size_t buf_size = 512;
   format_string = malloc(sizeof(char)*buf_size);
   if (strlen(fmt) > buf_size)
      perror("format string to long!");

   strncpy(format_string,fmt,strlen(fmt));
}

/*
 * 
 */
int create_read_from_line(Read* newRead, const char* current_line) {
   //printf("current line is %s\n",current_line);

   int entries_found = sscanf(current_line,line_format,&(newRead->id),
   &(newRead->chr),&(newRead->strand),newRead->seq,&(newRead->splitpos),&(newRead->size),
   newRead->prb,newRead->cal_prb,newRead->chastity,newRead->gene_id,&(newRead->p_start),
   &(newRead->exon_stop),&(newRead->exon_start),&(newRead->p_stop),&(newRead->true_cut));

   if (entries_found != 15) {
      return -1;
   }

   // make sequence lowercase but don't destroy brackets
   Py_ssize_t idx;
   for(idx=0;idx<strlen(newRead->seq);idx++) {
      if ( 65 <= newRead->seq[idx] && newRead->seq[idx] < 85)
         newRead->seq[idx] = newRead->seq[idx]+32;
   }

   if ( newRead->strand == 'D' )
      newRead->strand = '+';

   if ( newRead->strand == 'P' )
      newRead->strand = '-';

   return 0;
}


ParaParser::parseFile(const char* reads_filename) {

   // first define some constant strings

   FILE *reads_fs = fopen(reads_filename,"r");     

   if(reads_fs == NULL) {
      printf("Error: Could not open file: %s",reads_filename);
      exit(EXIT_FAILURE);
   }

   int reads_fid = fileno(reads_fs);
   struct stat reads_stat;
   if ( fstat(reads_fid,&reads_stat) == -1) {
      perror("fstat");
      exit(EXIT_FAILURE);
   }

   off_t reads_filesize = reads_stat.st_size;                                                                                                                                                                                  
   //printf("Reads file is of size %lu bytes\n",(unsigned long) reads_filesize);

   // ATTENTION this is an overestimator of the reads in the file
   // it is NOT the exact number
   int num_init_reads = reads_filesize / 200.0;

   read_array = malloc(sizeof(Read*)*num_init_reads);
   id_map = malloc(sizeof(unsigned long)*num_init_reads);

   //printf("Found %d reads.",numReads);

   // try to acquire file using mmap
   void *reads_area = mmap (NULL,reads_filesize,PROT_READ,MAP_PRIVATE,reads_fid,0);
   if (reads_area == MAP_FAILED) {
      perror("mmap");
      exit(EXIT_FAILURE);
   }

   close(reads_fid);
   printf("Successfully mapped %lu bytes of reads file into memory\n",(unsigned long)reads_filesize);
                                      
   char* lineBeginPtr = (char*) reads_area;
   char* lineEndPtr = (char*) reads_area;     
   char* end_of_mapped_area = ((char*) reads_area) + reads_filesize;

   while (*lineEndPtr != '\n' && lineEndPtr != end_of_mapped_area) lineEndPtr++;
   lineEndPtr++;
         
   char* current_line = malloc(sizeof(char)*512);
   memset(current_line,0,512);

   unsigned long line_size = lineEndPtr - lineBeginPtr;
   strncpy(current_line,lineBeginPtr,line_size);                                                                                                                                                                               
   current_line[line_size] = '\0';

   int readCtr = 0;
   int status = 0;

   num_reads = 0;
   map_idx = 0;

   while(1) {
      if (strcmp(current_line,"") == 0) 
         break;

      status = set_item_from_line(current_line);
      if (status != 0 )
         printf("Error while parsing line (status=%d).",status);

      lineBeginPtr = lineEndPtr;                                                                                                                                                                                         
      while (*(char*)lineEndPtr != '\n' && lineEndPtr != end_of_mapped_area) lineEndPtr++;
      lineEndPtr++;

      current_line = strncpy(current_line,lineBeginPtr,lineEndPtr-lineBeginPtr);
      current_line[lineEndPtr-lineBeginPtr] = '\0';

      readCtr += 1;
   }

   // clean up
   status = munmap(reads_area,reads_filesize);                                                                                                                                                                                 
   if(status != 0)
      perror("munmap");

   free(current_line);

   return PyInt_FromLong(num_reads);
fetchEntry(int id);

~ParaParser();



