#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2008 Fabio De Bona
# Copyright (C) 2008 Max-Planck-Society

import math
import numpy.matlib
import QPalmaDP
import pdb
from Plif import *

def set_param_palma(param, settings):

   #print 'Setting parameters ...'
   
   min_intron_len = settings['min_intron_len'] 
   max_intron_len = settings['max_intron_len']

   min_svm_score = settings['min_svm_score'] 
   max_svm_score = settings['max_svm_score'] 

   h = Plf()
   d = Plf()
   a = Plf()

   qualityPlifs = [None]*settings['numQualPlifs']
   donSP       = settings['numDonSuppPoints']
   accSP       = settings['numAccSuppPoints']
   lengthSP    = settings['numLengthSuppPoints']
   mmatrixSP   = settings['matchmatrixRows']*settings['matchmatrixCols']
   totalQualSP = settings['totalQualSuppPoints']

   ####################
   # Gapfunktion
   ####################
   h.limits    = logspace(math.log(min_intron_len,10),math.log(max_intron_len,10),lengthSP)
   h.penalties = param[0:lengthSP].flatten().tolist()[0]
   #h.transform = '+1' 
   h.transform = '' 
   h.name      = 'h' 
   h.max_len   = max_intron_len
   h.min_len   = min_intron_len
   h.id        = 1 
   h.use_svm   = 0 
   h.next_id   = 0 

   ####################
   # Donorfunktion
   ####################
   d.limits    = linspace(min_svm_score,max_svm_score,donSP)
   d.penalties = param[lengthSP:lengthSP+donSP].flatten().tolist()[0]
   #d.transform = '+1' 
   d.transform = '' 
   d.name      = 'd' 
   d.max_len   = max_svm_score
   d.min_len   = min_svm_score
   d.id        = 1 
   d.use_svm   = 0 
   d.next_id   = 0 

   ####################
   # Acceptorfunktion
   ####################
   a.limits    = linspace(min_svm_score,max_svm_score,accSP)
   a.penalties = param[lengthSP+donSP:lengthSP+donSP+accSP].flatten().tolist()[0]
   #a.transform = '+1' 
   a.transform = '' 
   a.name      = 'a' 
   a.max_len   = max_svm_score
   a.min_len   = min_svm_score
   a.id        = 1 
   a.use_svm   = 0 
   a.next_id   = 0 

   ####################
   # Matchmatrix
   ####################
   mmatrix = numpy.matlib.mat(param[lengthSP+donSP+accSP:lengthSP+donSP+accSP+mmatrixSP])


   mmatrix.reshape(settings['matchmatrixRows'],settings['matchmatrixCols'])

   ####################
   # Quality Plifs
   ####################
   for idx in range(settings['numQualPlifs']):
      currentPlif = Plf()
      currentPlif.limits    = linspace(settings['min_qual'],settings['max_qual'],settings['numQualSuppPoints']) 
      begin                 = lengthSP+donSP+accSP+mmatrixSP+(idx*settings['numQualSuppPoints'])
      end                   = lengthSP+donSP+accSP+mmatrixSP+((idx+1)*settings['numQualSuppPoints'])

      currentPlif.penalties = param[begin:end].flatten().tolist()[0]
      currentPlif.transform = '' 
      currentPlif.name      = 'q' 

      currentPlif.max_len   = settings['max_qual'] 
      currentPlif.min_len   = settings['min_qual']

      qualityPlifs[idx] = currentPlif

   return h,d,a,mmatrix,qualityPlifs


if __name__ == '__main__':
   min_svm_score=-5
   max_svm_score=5
   print linspace(min_svm_score,max_svm_score,30) 
