# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2008 Fabio De Bona
# Copyright (C) 2008 Max-Planck-Society

import pdb
import sys
import os.path

from numpy.matlib import mat,zeros,ones,inf

import palma_utils as pu

# some useful constants

extended_alphabet = ['-','a','c','g','t','n','[',']']
alphabet          = ['-','a','c','g','t','n']

alignment_alphabet   = ['-','a','c','g','t','n','z']


def print_prediction(example):
   dna_array   = example['dna_array']
   read_array  = example['read_array']

   dna   = map(lambda x: alignment_alphabet[x],dna_array)
   read  = map(lambda x: alignment_alphabet[x],read_array)

   spliceAlign = example['spliceAlign']
   estAlign    = example['estAlign']

   line1,line2,line3 = pprint_alignment(spliceAlign, estAlign, dna, read)
   
   result = '%s\n%s\n%s' %(line1,line2,line3)
   return result

def calc_info(acc,don,exons,qualities):
   min_score = 100
   max_score = -100

   for elem in acc:
      for score in elem:
         if score != -inf:
            min_score = min(min_score,score)
            max_score = max(max_score,score)

   print 'Acceptors max/min: %f / %f' % (max_score,min_score)

   min_score = 100
   max_score = -100

   for elem in don:
      for score in elem:
         if score != -inf:
            min_score = min(min_score,score)
            max_score = max(max_score,score)

   print 'Donor max/min: %f / %f' % (max_score,min_score)

   min_score = 10000
   max_score = 0
   mean_intron_len = 0

   for elem in exons:
      intron_len = int(elem[1,0] - elem[0,1])
      mean_intron_len += intron_len
      min_score = min(min_score,intron_len)
      max_score = max(max_score,intron_len)

   mean_intron_len /= 1.0*len(exons)
   print 'Intron length max/min: %d / %d mean length %f' % (max_score,min_score,mean_intron_len)

   min_score = 10000
   max_score = 0
   mean_quality = 0

   for qVec in qualities:
      for elem in qVec:
         min_score = min(min_score,elem)
         max_score = max(max_score,elem)

   print 'Quality values max/min: %d / %d mean' % (max_score,min_score)


def calc_stat(Acceptor,Donor,Exons):
   maxAcc = -100
   minAcc = 100
   maxDon = -100
   minDon = 100

   acc_vec_pos = []
   acc_vec_neg = []
   don_vec_pos = []
   don_vec_neg = []

   for jdx in range(len(Acceptors)):
      currentExons = Exons[jdx]
      currentAcceptor = Acceptors[jdx]
      currentAcceptor = currentAcceptor[1:]
      currentAcceptor.append(-inf)
      currentDonor = Donors[jdx]

      for idx,elem in enumerate(currentAcceptor):
         if idx == (int(currentExons[1,0])-1): # acceptor site
            acc_vec_pos.append(elem)
         else:
            acc_vec_neg.append(elem)

      for idx,elem in enumerate(currentDonor):
         if idx == (int(currentExons[0,1])): # donor site
            don_vec_pos.append(elem)
         else:
            don_vec_neg.append(elem)

   acc_pos = [elem for elem in acc_vec_pos if elem != -inf]
   acc_neg = [elem for elem in acc_vec_neg if elem != -inf]
   don_pos = [elem for elem in don_vec_pos if elem != -inf]
   don_neg = [elem for elem in don_vec_neg if elem != -inf]

   for idx in range(len(Sequences)):
      acc = [elem for elem in Acceptors[idx] if elem != -inf]
      maxAcc = max(max(acc),maxAcc)
      minAcc = min(min(acc),minAcc)

      don = [elem for elem in Donors[idx] if elem != -inf]
      maxDon = max(max(don),maxDon)
      minDon = min(min(don),minDon)


def pprint_alignment(_newSpliceAlign,_newEstAlign, dna_array, est_array):
   (qStart, qEnd, tStart, tEnd, num_exons, qExonSizes, qStarts, qEnds, tExonSizes, tStarts, tEnds) =\
         pu.get_splice_info(_newSpliceAlign,_newEstAlign)

   t_strand = '+'
   translation = '-ACGTN_'    #how aligned est and dna sequence is displayed
                              #(gap_char, 5 nucleotides, intron_char) 
   comparison_char = '| '     #first: match_char, second: intron_char

   (exon_length, identity, ssQuality, exonQuality, comparison, qIDX, tIDX) =\
             pu.comp_identity(dna_array, est_array, t_strand, qStart, tStart, translation, comparison_char)

   first_identity = None
   last_identity = None 
   for i in range(len(comparison)):
    if comparison[i] == '|' and first_identity is None: 
      first_identity = i 
    if comparison[-i] == '|' and last_identity is None: 
      last_identity = len(comparison) - i - 1

   try:
      for idx in range(len(dna_array)):
         dna_array[idx] = translation[int(dna_array[idx])]
         est_array[idx] = translation[int(est_array[idx])]
   except:
      #pdb.set_trace()
      pass

   line1 = "".join(dna_array)
   line2 = comparison
   line3 = "".join(est_array)

   return line1,line2,line3


def get_alignment(_newSpliceAlign,_newEstAlign, dna_array, est_array):
   return pu.get_splice_info(_newSpliceAlign,_newEstAlign)


##########

def combine_files(partial_files,new_file):
   try:
      fh = open(new_file,'w')
   except IOError:
      print 'Problem while trying to open file: %s' % new_file
      sys.exit(1)

   for pfile in partial_files:
      for line in open(pfile):
         fh.write(line)

   fh.close()


def split_file(filename,result_dir,num_parts):
   """
   Splits a file of n lines into num_parts parts
   """

   jp = os.path.join

   all_intervals = []

   print 'counting lines'
   line_ctr = 0
   for line in open(filename,'r'):
      line_ctr += 1

   print 'found %d lines' % line_ctr

   part_size = line_ctr / num_parts
   begin = 0
   end = 0

   for idx in range(1,num_parts+1):
      begin = end

      if idx == num_parts:
         end = line_ctr
      else:
         end = begin+part_size+1

      all_intervals.append((begin,end))

   parts_fn = []
   for pos,params in enumerate(all_intervals):
      beg,end = params
      out_fn = jp(result_dir,'map.part_%d'%pos)
      print out_fn
      parts_fn.append(out_fn)
      out_fh = open(out_fn,'w+')

   parts_fn.reverse()
   all_intervals.reverse()

   lineCtr = 0
   beg = -1
   end = -1
   in_fh = open(filename,'r')
   while True:
      line = in_fh.readline()
      if line == '':
         break

      if beg <= lineCtr < end:
         out_fh.write(line)
         lineCtr += 1
      else:
         (beg,end) = all_intervals.pop()
         out_fn = parts_fn.pop()
         out_fh.close()
         out_fh = open(out_fn,'w+')
         out_fh.write(line)

   out_fh.close()


def get_slices(dataset_size,num_nodes):
   """
   Given the size of the dataset and the number of nodes which shall be used
   this function returns the slice of the dataset for each node.
   """

   all_instances = []

   part = dataset_size / num_nodes
   begin = 0
   end = 0
   for idx in range(1,num_nodes+1):
      
      if idx == num_nodes:
         begin = end
         end   = dataset_size
      else:
         begin = end
         end = begin+part

      params = (begin,end)

      all_instances.append(params)

   return all_instances


if __name__ == '__main__':
   split_file('/fml/ag-raetsch/home/fabio/tmp/lyrata_analysis/map.vm','/tmp',25)
