#/bin/bash

#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2007-2009 Gunnar Rätsch, Fabio De Bona
# Copyright (C) 2007-2009 Max Planck Society 
#

set -e 

. ./bin/qpalma_config.sh

echo QPalma setup script \(version 1.0\) 
echo ===================================
echo

echo QPalma base directory \(currently set to \"$QPALMA_PATH\", suggest to set to \"`pwd`\"\)
read QPALMA_PATH       
echo '=>' Setting QPALMA base directory to \"$QPALMA_PATH\"
echo

QPALMA_BIN_PATH=${QPALMA_PATH}/bin
QPALMA_TOOLS_PATH=${QPALMA_PATH}/tools
QPALMA_MODULES_PATH=${QPALMA_PATH}/modules

echo Which optimizer should be used:
echo 1. Mosek \(commercial optimizer with available trial license\)
echo 2. CVXOPT \(free optimizer\)
read OPTIMIZER_CHOICE
if [ "$OPTIMIZER_CHOICE" != '1' -a "$OPTIMIZER_CHOICE" != '2'  ];
then
	echo Unrecognized choice: \"$OPTIMIZER_CHOICE\"
	echo Aborting
	false
fi

if [ "$OPTIMIZER_CHOICE" == '1' ];
then
    OPTIMIZER=mosek

    echo The installer script can try to download the binaries of the optimization package MOSEK
    echo together with a trial license. You choose among the following options:
    echo 1. Use existing installation
    echo 2. Download Linux 32-bit x86 binaries
    echo 3. Download Linux 64-bit x86 binaries
    echo 4. Download Mac OS X 32-bit x86 binaries
    #echo 5. Download Mac OS X 32-bit PPC binaries
    read MOSEK_CHOICE

    if [ "$MOSEK_CHOICE" != '1' -a "$MOSEK_CHOICE" != '2' -a "$MOSEK_CHOICE" != '3' -a "$MOSEK_CHOICE" != '4' -a "$MOSEK_CHOICE" != '5' ];
    then
	echo Unrecognized choice: \"$MOSEK_CHOICE\"
	echo Aborting
	false
    fi
    
    if [ "$MOSEK_CHOICE" == '1' ];
    then
	echo Mosek installation bin directory \(currently set to \"$OPTIMIZER_PATH\"\)
	    read OPTIMIZER_PATH
	    echo '=>' Setting Mosek bin directory to \"$OPTIMIZER_PATH\"
    else
	cd modules
	if [ "$MOSEK_CHOICE" == '2' ];
	then
	    echo '=>' Downloading Mosek for Linux 32-bit x86
	    curl http://www.mosek.com/fileadmin/download/stable/5/mosektoolslinux32x86.tar.gz > mosektoolslinux32x86.tar.gz
	    echo -n uncompressing...
	    tar xzf mosektoolslinux32x86.tar.gz
	    OPTIMIZER_PATH=${QPALMA_MODULES_PATH}/mosek/5/tools/platform/linux32x86
	fi
	if [ "$MOSEK_CHOICE" == '3' ];
	then
	    echo '=>' Downloading Mosek for Linux 64-bit x86
	    curl http://www.mosek.com/fileadmin/download/stable/5/mosektoolslinux64x86.tar.gz > mosektoolslinux64x86.tar.gz
	    echo -n uncompressing... 
	    tar xzf mosektoolslinux64x86.tar.gz
	    OPTIMIZER_PATH=${QPALMA_MODULES_PATH}/mosek/5/tools/platform/linux64x86
	fi
	if [ "$MOSEK_CHOICE" == '4' ];
	then
	    echo '=>' Downloading Mosek for Mac OS X 32-bit x86
	    curl http://www.mosek.com/fileadmin/download/stable/5/mosektoolsosx32x86.tar.gz > mosektoolsosx32x86.tar.gz
	    echo -n uncompressing...
	    tar xzf mosektoolsosx32x86.tar.gz
	    OPTIMIZER_PATH=${QPALMA_MODULES_PATH}/mosek/5/tools/platform/osx32x86
	fi
	if [ "$MOSEK_CHOICE" == '5' ];
	then
	    echo '=>' Downloading Mosek for Mac OS X 32-bit PPC
	    curl http://www.mosek.com/fileadmin/download/stable/5/mosektoolsosx32ppc.tar.gz > mosektoolsosx32ppc.tar.gz
	    echo -n uncompressing...
	    tar xzf mosektoolsosx32ppc.tar.gz
	    OPTIMIZER_PATH=${QPALMA_MODULES_PATH}/mosek/5/tools/platform/osx32ppc
	fi
	echo Done.
	cd ..
	echo 
	echo The Mosek installation comes with a time-limited trial license. 
	echo \(Check the license file $OPTIMIZER_PATH/../../licenses/mosek.lic\)
	echo Please consider purchasing a license after trying.
    fi
fi

if [ "$OPTIMIZER_CHOICE" == '2' ];
then
    OPTIMIZER=CVXOPT
    echo
    echo To install CVXOPT, download it from 
    echo   http://abel.ee.ucla.edu/cvxopt/download/index.html 
    echo and follow the instructions.
    echo
    echo CVXOPT installation bin directory \(currently set to \"$OPTIMIZER_PATH\"\)
    read OPTIMIZER_PATH
    echo '=>' Setting CVXOPT bin directory to \"$OPTIMIZER_PATH\"
fi

echo
echo Compiling modules:
echo
echo 1. Parameter Parser Module
echo --------------------------
cd modules/ParaParser
make 
cd ../..
echo

echo 2. Dynamic Programming Module
echo -----------------------------
cd modules/DynProg
make 
cd ../..
echo

echo 3. Interval Query Module
echo ------------------------
cd modules/IntervalQuery
make 
cd ../..
echo

echo Writing config file bin/qpalma_config.sh

cp bin/qpalma_config.sh bin/qpalma_config.sh.bak
grep -v -e QPALMA_PATH -e QPALMA_MODULES_PATH -e OPTIMIZER_PATH -e OPTIMIZER bin/qpalma_config.sh.bak > bin/qpalma_config.sh

# appending the relevant lines to qpalma_config.sh
echo export QPALMA_PATH=$QPALMA_PATH >> bin/qpalma_config.sh
echo export QPALMA_BIN_PATH=$QPALMA_BIN_PATH >> bin/qpalma_config.sh
echo export QPALMA_TOOLS_PATH=$QPALMA_TOOLS_PATH >> bin/qpalma_config.sh
echo export QPALMA_MODULES_PATH=$QPALMA_MODULES_PATH >> bin/qpalma_config.sh
echo export OPTIMIZER=$OPTIMIZER >> bin/qpalma_config.sh
echo export OPTIMIZER_PATH=$OPTIMIZER_PATH >> bin/qpalma_config.sh

(cd galaxy; ./make.sh ; cd ..)

echo
echo QPalma setup done.
echo 
