#!/bin/bash


PROG=`basename $0`
DIR=`dirname $0`
QPALMA_DIR=${DIR}/../

. $QPALMA_DIR/bin/qpalma_config.sh

. $QPALMA_BIN_PATH/qpalma_header.sh $0

if [ "$#" = "0" ];
then
	echo usage $0 \<genome-fasta\> \<read-fastq\> \<annotation-gff3\> \<gff3-source\> \<acc-splice-pred\> \<don-splice-pred\>
	exit -1
fi

set -e 



GENOME=$1
fastq=$2

fastq_sample=${fastq}.read_sample.fastq
fastq_working=${fastq}.working
mismatch=${fastq}.read_sample.mismatch.pickle
quality_sample=${fastq}.quality_sample

read_length=`python $QPALMA_TOOLS_PATH/sequences/read_length.py $fastq 1000`
echo Determined read length: $read_length
echo -----------------------------

num_genes=500
num_reads=10
qualitysamplesize=1000000 
errormodelsamplesize=100000

mkdir -p $fastq_working

if [ ! -f $fastq_sample ];
	then
	echo Subsampling original read file to $qualitysamplesize reads
	echo ---------------------------------------------------------
	echo python $QPALMA_TOOLS_PATH/sequences/subsample_fastq.py $fastq $qualitysamplesize $fastq_sample
	python $QPALMA_TOOLS_PATH/sequences/subsample_fastq.py $fastq $qualitysamplesize $fastq_sample
else
	echo Subsampled read file $fastq_sample already exists
fi 

if [ ! -f $mismatch ];
	then
	echo Estimating error model based on $errormodelsamplesize reads and their alignment to the reference genome
	echo --------------------------------------------------------------------------------------------------
	echo $QPALMA_TOOLS_PATH/errormodel/create_error_model.sh $fastq_sample $GENOME $errormodelsamplesize
	$QPALMA_TOOLS_PATH/errormodel/create_error_model.sh $fastq_sample $GENOME $errormodelsamplesize $mismatch $quality_sample
else
	echo Error model $mismatch already exists
fi

echo Creating artificial reads based on annotated transcripts for training QPALMA
echo ----------------------------------------------------------------------------
echo $QPALMA_TOOLS_PATH/gff/create_art_reads.sh $fastq_working $mismatch $GENOME $3 $4 $read_length $num_genes $num_reads "" $quality_sample
$QPALMA_TOOLS_PATH/gff/create_art_reads.sh $fastq_working $mismatch $GENOME $3 $4 $read_length $num_genes $num_reads "" $quality_sample

export QPALMA_C=10
export QPALMA_half_window_size=25000
export QPALMA_prb_offset=33
export QPALMA_iterations=50
export QPALMA_max_intron_length=50000
export QPALMA_max_qual=40
export QPALMA_num_supp_points=10
export QPALMA_enable_intron_length=False
export QPALMA_genome_dir=$fastq_working/genome
export QPALMA_acceptor_scores=$5
export QPALMA_donor_scores=$6

echo Starting QPALMA Training 
echo ------------------------
echo $QPALMA_PATH/galaxy/qpalma_train.sh $fastq_working/qpalma_train.bed $fastq_working/qpalma $fastq_working/qpalma/parameters.qpalma
$QPALMA_PATH/galaxy/qpalma_train.sh $fastq_working/qpalma_train.bed $fastq_working/qpalma $fastq_working/qpalma/parameters.qpalma

echo Done.

cp $fastq_working/qpalma/parameters.qpalma parameters.qpalma
#rm -rf $fastq_working

echo Resulting QPALMA parameters are in parameters.qpalma
echo ----------------------------------------------------

echo 
echo Done.
